# rhpkg bash completion

_rhpkg()
{
    COMPREPLY=()

    in_array()
    {
        local i
        for i in $2; do
            [[ $i = $1 ]] && return 0
        done
        return 1
    }

    _filedir_exclude_paths()
    {
        _filedir "$@"
        for ((i=0; i<=${#COMPREPLY[@]}; i++)); do
            [[ ${COMPREPLY[$i]} =~ /?\.git/? ]] && unset COMPREPLY[$i]
        done
    }

    local cur prev
    # _get_comp_words_by_ref is in bash-completion >= 1.2, which EL-5 lacks.
    if type _get_comp_words_by_ref &>/dev/null; then
        _get_comp_words_by_ref cur prev
    else
        cur="${COMP_WORDS[COMP_CWORD]}"
        prev="${COMP_WORDS[COMP_CWORD-1]}"
    fi

    # global options

    local options="--help -v -q"
    local options_value="--config --dist --user --path --released --unreleased"
    local commands="build bugzilla brew-path chain-build ci clean clog \
    clone co commit compile container-build covscan diff errata gimmespec \
    giturl help gitbuildhash import install lint local maven-build mockbuild \
    mock-config module-build module-build-cancel module-build-info \
    module-build-local module-build-watch module-overview module-scratch-build \
    new new-sources \
    patch prep publican-build pull push remote retire rpmdiff scratch-build \
    sources srpm switch-branch tag tests unused-patches upload verify-files \
    verrel wrapper-rpm"

    # parse main options and get command

    local command=
    local command_first=
    local path=

    local i w
    for (( i = 0; i < ${#COMP_WORDS[*]} - 1; i++ )); do
        w="${COMP_WORDS[$i]}"
        # option
        if [[ ${w:0:1} = - ]]; then
            if in_array "$w" "$options_value"; then
                ((i++))
                [[ "$w" = --path ]] && path="${COMP_WORDS[$i]}"
            fi
        # command
        elif in_array "$w" "$commands"; then
            command="$w"
            command_first=$((i+1))
            break
        fi
    done

    # complete base options

    if [[ -z $command ]]; then
        if [[ $cur == -* ]]; then
            COMPREPLY=( $(compgen -W "$options $options_value" -- "$cur") )
            return 0
        fi

        case "$prev" in
            --config)
                _filedir_exclude_paths
                ;;
            --dist)
                ;;
            --user|-u)
                ;;
            --path)
                _filedir_exclude_paths
                ;;
            --released)
                ;;
            --unreleased)
                ;;
            *)
                COMPREPLY=( $(compgen -W "$commands" -- "$cur") )
                ;;
        esac

        return 0
    fi

    # parse command specific options

    local options=
    local options_target= options_arch= options_branch= options_string= options_file= options_dir= options_srpm= options_mroot=
    local options_yaml=
    local after= after_more=

    case $command in
        help|brew-path|gimmespec|gitbuildhash|giturl|lint|new|push|unused-patches|verrel)
            ;;
        bugzilla)
            options="--username --password --fixed-in --modified --comment \
                     --last-commit --since-last-build --since-commit \
                     --since-build --bug --dry-run"
            ;;
        build)
            options="--nowait --background --skip-tag --scratch --fast --md5 \
                     --test-type --test-tier --skip-remote-rules-validation"
            options_arch="--arches"
            options_srpm="--srpm"
            options_target="--target"
            ;;
        chain-build)
            options="--nowait --background --md5 --test-type --test-tier"
            options_target="--target"
            options_arch="--arches"
            after="package"
            after_more=true
            ;;
        clean)
            options="--dry-run -x"
            ;;
        clog)
            options="--raw"
            ;;
        clone|co)
            options="--branch -b --branches -B --anonymous"
            after="package"
            ;;
        commit|ci)
            options="--push --clog --raw --tag --signoff"
            options_string="--message"
            options_file="--file"
            after="file"
            after_more=true
            ;;
        compile|install)
            options="--short-circuit"
            options_arch="--arch"
            options_dir="--builddir"
            ;;
        container-build)
            options="--scratch --nowait --skip-remote-rules-validation"
            options_target="--target"
            options_string="--repo-url"
            ;;
        covscan)
            options="--check-patches --covscan-args"
            options_mroot="--mock-profile -r"
            options_srpm="--base"
            ;;
        diff)
            options="--cached"
            after="file"
            after_more=true
            ;;
        errata)
            options="--erratum --product-version qe newfiles --dry-run, --message \
                     --bugzilla-username --bugzilla-password \
                     new-state add-bugs remove-bugs add-builds \
                     --last-commit --since-last-build --since-commit --since-build \
                     --bug --nowait-rpmdiff"
            ;;
        import)
            options="--create --skip-diffs"
            options_branch="--branch"
            after="srpm"
            ;;
        lint)
            options="--info"
            options_file="--rpmlintconf"
            ;;
        local)
            options="--md5"
            options_arch="--arch"
            options_dir="--builddir"
            ;;
        maven-build)
            options="--nowait --background --skip-tag --scratch --sources \
                     --patches --goal --profile --property --env --package \
                     --jvm-option --maven-option --specfile --debug --md5 \
                     --test-type --test-tier"
            options_file="--ini"
            options_arch="--arches"
            options_target="--target"
            ;;
        maven-chain)
            options="--md5 --nowait --background --debug --skip-tag --scratch \
                     --test-type --test-tier"
            options_arch="--arches"
            options_target="--target"
            options_file="--ini"
            ;;
        mock-config)
            options_target="--target"
            options_arch="--arch"
            ;;
        mockbuild)
            options="--md5 --no-clean --no-cleanup-after --no-clean-all"
            options_mroot="--root"
            ;;
        module-build)
            options="--scratch --watch"
            options_string="--optional --requires --buildrequires"
            options_yaml="--file"
            options_srpm="--srpm"
            ;;
        module-build-local)
            options="--skip-tests"
            options_string="--add-local-build --stream --set-default-stream"
            options_yaml="--file"
            options_srpm="--srpm"
            ;;
        module-overview)
            options="--unfinished"
            options_string="--limit"
            ;;
        module-scratch-build)
            options="--watch"
            options_string="--optional --requires --buildrequires"
            options_yaml="--file"
            options_srpm="--srpm"
            ;;
        patch)
            options="--rediff"
            options_string="--suffix"
            ;;
        prep|verify-files)
            options_arch="--arch"
            options_dir="--builddir"
            ;;
        publican-build)
            options="--scratch --nowait"
            options_string="--message --publican-args"
            options_branch="--branch"
            options_file="--publican-config"
            options_dir="--lang"
            options_target="--target"
            ;;
        pull)
            options="--rebase --no-rebase"
            ;;
        retire)
            options="--push"
            after_more=true
            ;;
        rpmdiff)
            options="--base -b"
            ;;
        remote)
            ;;
        scratch-build)
            options="--nowait --background --md5 --test-type --test-tier"
            options_target="--target"
            options_arch="--arches"
            options_srpm="--srpm"
            ;;
        sources)
            options_dir="--outdir"
            ;;
        srpm)
            options="--md5"
            ;;
        switch-branch)
            options="--list --fetch"
            after="branch"
            ;;
        tag)
            options="--clog --raw --force --list --delete"
            options_string="--message"
            options_file="--file"
            after_more=true
            ;;
        tests)
            options="--branch -b --anonymous"
            after="package"
            ;;
        upload|new-sources)
            after="file"
            after_more=true
            ;;
        verify-files)
            options_dir="--builddir"
            options_arch="--arch"
            ;;
        wrapper-rpm)
            options="--nowait --background --skip-tag --scratch --build \
                     --specfile --no-create-build --md5 --test-type --test-tier"
            options_target="--target"
            options_arch="--arches"
            ;;
    esac

    local all_options="--help $options"
    local all_options_value="$options_target $options_arch $options_branch $options_string \
    $options_file $options_dir $options_srpm $options_mroot $options_yaml"

    # count non-option parametrs

    local i w
    local last_option=
    local after_counter=0
    for (( i = $command_first; i < ${#COMP_WORDS[*]} - 1; i++)); do
        w="${COMP_WORDS[$i]}"
        if [[ ${w:0:1} = - ]]; then
            if in_array "$w" "$all_options"; then
                last_option="$w"
                continue
            elif in_array "$w" "$all_options_value"; then
                last_option="$w"
                ((i++))
                continue
            fi
        fi
        in_array "$last_option" "$options_arch" || ((after_counter++))
    done

    # completion
    if [[ -n $options_target ]] && in_array "$prev" "$options_target"; then
        COMPREPLY=( $(compgen -W "$(_rhpkg_target)" -- "$cur") )

    elif [[ -n $options_arch ]] && in_array "$last_option" "$options_arch"; then
        COMPREPLY=( $(compgen -W "$(_rhpkg_arch) $all_options" -- "$cur") )

    elif [[ -n $options_srpm ]] && in_array "$prev" "$options_srpm"; then
        _filedir_exclude_paths "*.src.rpm"

    elif [[ -n $options_yaml ]] && in_array "$prev" "$options_yaml"; then
        _filedir_exclude_paths "yaml"

    elif [[ -n $options_branch ]] && in_array "$prev" "$options_branch"; then
        COMPREPLY=( $(compgen -W "$(_rhpkg_branch "$path")" -- "$cur") )

    elif [[ -n $options_file ]] && in_array "$prev" "$options_file"; then
        _filedir_exclude_paths

    elif [[ -n $options_dir ]] && in_array "$prev" "$options_dir"; then
        _filedir_exclude_paths -d

    elif [[ -n $options_mroot ]] && in_array "$prev" "$options_mroot"; then
        COMPREPLY=( $(compgen -W "$(_rhpkg_mroot)" -- "$cur") )

    elif [[ -n $options_string ]] && in_array "$prev" "$options_string"; then
        COMPREPLY=( )

    else
        local after_options=

        if [[ $after_counter -eq 0 ]] || [[ $after_more = true ]]; then
            case $after in
                file)    _filedir_exclude_paths ;;
                srpm)    _filedir_exclude_paths "*.src.rpm" ;;
                branch)  after_options="$(_rhpkg_branch "$path")" ;;
                package) after_options="$(_rhpkg_package "$cur")";;
            esac
        fi

        if [[ $cur != -* ]]; then
            all_options=
            all_options_value=
        fi

        COMPREPLY+=( $(compgen -W "$all_options $all_options_value $after_options" -- "$cur" ) )
    fi

    return 0
} &&
complete -F _rhpkg rhpkg

_rhpkg_target()
{
    brew list-targets --quiet 2>/dev/null | cut -d" " -f1
}

_rhpkg_arch()
{
    echo "i386 x86_64 ppc ppc64 s390 s390x sparc sparc64"
}

_rhpkg_branch()
{
    local git_options= format="--format %(refname:short)"
    [[ -n $1 ]] && git_options="--git-dir=$1/.git"

    git $git_options for-each-ref $format 'refs/remotes/origin/*' \
        | sed 's,origin/,,'
    git $git_options for-each-ref $format 'refs/heads'
}

_rhpkg_mroot()
{
    ls /etc/mock |grep .cfg |grep -v default|sed -r 's/.cfg//'
}

_rhpkg_package()
{
    repoquery -C --qf=%{sourcerpm} "$1*" 2>/dev/null | sort -u | sed -r 's/(-[^-]*){2}\.src\.rpm$//'
}

# Local variables:
# mode: shell-script
# sh-basic-offset: 4
# sh-indent-comment: t
# indent-tabs-mode: nil
# End:
# ex: ts=4 sw=4 et filetype=sh
