# -*- coding: utf-8 -*-
"""
Copyright (C) 2017 Red Hat Inc.
Author(s): Chenxiong Qi <cqi@redhat.com>

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
the full text of the license.
"""

"""
This module defines the mappings from Fedora and RHEL branches to RPM defines.

Both of the mappings are a list of 2-elements tuples, where the first one is
regex pattern to match branch name, and the second is a string format for
formatting final dist. Currently, dist format accepts only one data distval.

In the future, when new mapping is required, just add new one in following two
mapping lists individually.

IMPORTANT:

The order of the mappings is important. The first match wins from top to
bottom. If you are adding a really generic mapping, please consider to add it
at the end.
"""


from_rhel_branch = [
    # Fields: branch_regex, disttag_fmt

    # JBoss Core Services
    (r'jb-cs(-.+)?-rhel-\d$', 'jbcs.el%(distval)s'),

    # Check if this is a jboss eap thingy
    # Ok, their tags are .ep#.el#
    # Grabbing a magic indice from the string, probably not the best
    # but it's quick and dirty.
    (r'jb-eap-6-rhel-\d$', 'ep6.el%(distval)s'),
    (r'jb-eap-7-rhel-\d$', 'ep7.el%(distval)s'),

    (r'eso-rhel-6', 'el6eso'),
    (r'eng-epel-\d', 'el%(distval)sengepel'),
    (r'eng-rhel-\d', 'el%(distval)seng'),
    (r'sed-rhel-\d', 'el%(distval)ssed'),
    (r'satellite', 'el%(distval)ssat'),

    # Now check for Red Hat OpenStack
    # Grizzly was special
    (r'rh-grizzly', 'el%(distval)sost'),
    (r'ruby193-rh-grizzly', 'el%(distval)sost'),

    # RHOS 4 and later use rhos-X-rhel-Y
    (r'rhos-', 'el%(distval)sost'),
    (r'ruby193-rhos-', 'el%(distval)sost'),

    # Now check for Red Hat Storage (RHS)
    # Original RHS rebranded to Red Hat Gluster Storage
    (r'rhg?s-.*rhel-6', 'el6rhs'),
    (r'rhg?s-', 'el%(distval)srhgs'),

    (r'.*rhel-7.0-aarch64.*', 'sa1.1'),

    # Check for rhevm
    (r'.*rhevm-\d(\.\d+)?-rhel-[567](\.\d+)?$', 'el%(distval)sev'),

    (r'ceph-', 'el%(distval)scp'),

    # Ansible Runner
    (r'ansible-runner', 'el%(distval)sar'),

    # Now check for Ansible Tower
    (r'ansible-tower', 'el%(distval)sat'),

    # Now check for Ansible Engine.
    # Note: this rule has to be after ansible-tower and ansible-runner so we
    # won't eat their branch prefix.
    (r'ansible', 'el%(distval)sae'),

    (r'discovery-\d\.\d+-rhel-\d', 'el%(distval)sd'),
]


from_fedora_branch = [
    # Fields: branch_regex, disttag_fmt

    (r'eng-fedora-\d+', 'fc%(distval)seng'),
]
