# -*- coding: utf-8 -*-
"""
Interface for remote services

Copyright (C) 2014 Red Hat Inc.
Author(s): Tomas Tomecek <ttomecek@redhat.com>

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
the full text of the license.
"""

import logging


class LoggingService(object):
    def __init__(self, log=None, verbose=False, debug=False, *args, **kwargs):
        self.debug = debug
        if log is None:
            if debug:
                level = logging.DEBUG
            elif verbose:
                level = logging.INFO
            else:
                level = logging.WARNING
            logging.basicConfig(format='%(message)s', level=level)
            self.log = logging
        else:
            self.log = log


class DryRunService(object):
    """
    Service supports dry-run -- do not change anything, just write possible actions to stdout
    """

    def __init__(self, dry_run=False, *args, **kwargs):
        self.dry_run = dry_run
        if self.dry_run:
            self.dry_logger = logging.getLogger('dry_logger')
            self.dry_logger.setLevel(logging.INFO)
            # getLogger always returns the same instance
            # make sure that the handler is added only once otherwise we will get duplicate messages
            if not self.dry_logger.handlers:
                ch = logging.StreamHandler()
                ch.setLevel(logging.INFO)
                formatter = logging.Formatter('%(message)s')
                ch.setFormatter(formatter)
                self.dry_logger.addHandler(ch)

    def dry_log(self, *args):
        """ If dry-run is enabled, return True """
        if self.dry_run:
            self.dry_logger.info(*args)
            return True
        return False
